diff --git a/Docker/lofar-ci/Dockerfile_ci_sas b/Docker/lofar-ci/Dockerfile_ci_sas
index 8c03dbfab9bd3e22c96eb7513a3511f7f131b5c6..b515298af20f9d3a3bd01d36e1628ac2eec8c2c5 100644
--- a/Docker/lofar-ci/Dockerfile_ci_sas
+++ b/Docker/lofar-ci/Dockerfile_ci_sas
@@ -16,7 +16,7 @@ RUN yum erase -y postgresql postgresql-server postgresql-devel && \
     cd /bin && ln -s /usr/pgsql-9.6/bin/initdb && ln -s /usr/pgsql-9.6/bin/postgres
 ENV PATH /usr/pgsql-9.6/bin:$PATH 
 
-RUN pip3 install cython kombu lxml requests pygcn xmljson mysql-connector-python python-dateutil Django==3.0.9 djangorestframework==3.11.1 djangorestframework-xml ldap==1.0.2 flask fabric coverage python-qpid-proton PyGreSQL numpy h5py psycopg2 testing.postgresql Flask-Testing scipy Markdown django-filter python-ldap python-ldap-test ldap3 django-jsonforms django-json-widget django-jsoneditor drf-yasg flex swagger-spec-validator django-auth-ldap mozilla-django-oidc jsonschema comet pyxb==1.2.5 graphviz isodate astropy astroplan packaging django-debug-toolbar pymysql
+RUN pip3 install cython kombu lxml requests pygcn xmljson mysql-connector-python python-dateutil Django==3.0.9 djangorestframework==3.11.1 djangorestframework-xml ldap==1.0.2 flask fabric coverage python-qpid-proton PyGreSQL numpy h5py psycopg2 testing.postgresql Flask-Testing scipy Markdown django-filter python-ldap python-ldap-test ldap3 django-jsonforms django-json-widget django-jsoneditor drf-yasg flex swagger-spec-validator django-auth-ldap mozilla-django-oidc jsonschema comet pyxb==1.2.5 graphviz isodate astropy packaging django-debug-toolbar pymysql astroplan
 
 #Viewflow package 
 RUN pip3 install django-material django-viewflow
diff --git a/SAS/TMSS/services/scheduling/lib/constraints/template_constraints_v1.py b/SAS/TMSS/services/scheduling/lib/constraints/template_constraints_v1.py
index b6486489364ac6d5d6da26a100983ecd49f37469..28e6ccdd4ba47cf49f6f6bd4e3646108bcdcd6fc 100644
--- a/SAS/TMSS/services/scheduling/lib/constraints/template_constraints_v1.py
+++ b/SAS/TMSS/services/scheduling/lib/constraints/template_constraints_v1.py
@@ -30,7 +30,7 @@ logger = logging.getLogger(__name__)
 from datetime import datetime, timedelta
 
 from lofar.sas.tmss.tmss.tmssapp import models
-from lofar.sas.tmss.tmss.tmssapp.conversions import LOFAR_CENTER_OBSERVER, sun_rise_and_set_at_lofar_center, Time
+from lofar.sas.tmss.tmss.tmssapp.conversions import create_astroplan_observer_for_station, Time, timestamps_and_stations_to_sun_rise_and_set
 
 from . import ScoredSchedulingUnit
 
@@ -67,6 +67,7 @@ def can_run_within_timewindow_with_daily_constraints(scheduling_unit: models.Sch
             timestamps.append(timestamps[-1] + timedelta(hours=8))
         timestamps.append(possible_stop_time)
 
+        LOFAR_CENTER_OBSERVER = create_astroplan_observer_for_station('CS002')
         if constraints['daily']['require_night'] and all(LOFAR_CENTER_OBSERVER.is_night(timestamp) for timestamp in timestamps):
             return True
 
@@ -101,9 +102,15 @@ def get_earliest_possible_start_time(scheduling_unit: models.SchedulingUnitBluep
 
     try:
         if constraints['daily']['require_day'] or constraints['daily']['require_night']:
-            sun_rise, sun_set = sun_rise_and_set_at_lofar_center(lower_bound)
 
+            # TODO: TMSS-254 and TMSS-255
             # TODO: take avoid_twilight into account
+            # for now, use the incorrect proof of concept which works for the demo
+            # but... this should be rewritten completely using Joerns new sun_events
+            LOFAR_CENTER_OBSERVER = create_astroplan_observer_for_station('CS002')
+            sun_events = timestamps_and_stations_to_sun_rise_and_set(timestamps=[lower_bound], stations=['CS002'])['CS002']
+            sun_set = sun_events['sunset'][0]['start']
+            sun_rise = sun_events['sunrise'][0]['end']
             if constraints['daily']['require_day']:
                 if lower_bound+scheduling_unit.duration > sun_set:
                     return LOFAR_CENTER_OBSERVER.sun_rise_time(time=Time(sun_set), which='next').to_datetime()
@@ -117,10 +124,6 @@ def get_earliest_possible_start_time(scheduling_unit: models.SchedulingUnitBluep
                 if lower_bound >= sun_set:
                     return lower_bound
                 return sun_set
-
-            # if constraints['daily']['require_night']:
-            #     # for now, just assume next_sun_set is fine (leaving lots of open gaps)
-            #     return next_sun_set
     except Exception as e:
         logger.exception(str(e))
 
diff --git a/SAS/TMSS/src/tmss/settings.py b/SAS/TMSS/src/tmss/settings.py
index ec4f811934c976a158763404d6d039b867ccb4e2..8fecfed613d005bf62e685008744ebb9b45b7dea 100644
--- a/SAS/TMSS/src/tmss/settings.py
+++ b/SAS/TMSS/src/tmss/settings.py
@@ -97,7 +97,6 @@ INSTALLED_APPS = [
 
 MIDDLEWARE = [
     'django.middleware.gzip.GZipMiddleware',
-    'debug_toolbar.middleware.DebugToolbarMiddleware',
     'django.middleware.security.SecurityMiddleware',
     'django.contrib.sessions.middleware.SessionMiddleware',
     'django.middleware.common.CommonMiddleware',
diff --git a/SAS/TMSS/src/tmss/tmssapp/conversions.py b/SAS/TMSS/src/tmss/tmssapp/conversions.py
index 8666a8bc54e96bbb166ea7facb4f23672a034c83..ce112f7b30b8f697baf91d4da9202899703715ba 100644
--- a/SAS/TMSS/src/tmss/tmssapp/conversions.py
+++ b/SAS/TMSS/src/tmss/tmssapp/conversions.py
@@ -3,22 +3,60 @@ import astropy.units
 from lofar.lta.sip import station_coordinates
 from datetime import datetime
 from astropy.coordinates.earth import EarthLocation
+from astropy.coordinates import Angle
 from astroplan.observer import Observer
 
 
-# astropy/astroplan constants for lofar
-LOFAR_CENTER_COORDS = station_coordinates.parse_station_coordinates()["CS002_LBA"]
-LOFAR_CENTER_LOCATION = EarthLocation.from_geocentric(x=LOFAR_CENTER_COORDS['x'], y=LOFAR_CENTER_COORDS['y'], z=LOFAR_CENTER_COORDS['z'],  unit=astropy.units.m)
-LOFAR_CENTER_OBSERVER = Observer(LOFAR_CENTER_LOCATION, name="LOFAR", timezone="UTC")
+def create_astroplan_observer_for_station(station: str) -> Observer:
+    '''
+    returns an astroplan observer for object for a given station, located in the LBA center of the given station
+    :param station: a station name, e.g. "CS002"
+    :return: astroplan.observer.Observer object
+    '''
 
+    coords = station_coordinates.parse_station_coordinates()["%s_LBA" % station.upper()]
+    location = EarthLocation.from_geocentric(x=coords['x'], y=coords['y'], z=coords['z'],  unit=astropy.units.m)
+    observer = Observer(location, name="LOFAR", timezone="UTC")
+    return observer
 
-def sun_rise_and_set_at_lofar_center(timestamp: datetime) -> (datetime, datetime):
-    '''compute the sunrise and sunset at the lofar center'''
-    sun_rise = LOFAR_CENTER_OBSERVER.sun_rise_time(time=Time(timestamp), which='previous')
-    if sun_rise.to_datetime().date() < timestamp.date():
-        sun_rise = LOFAR_CENTER_OBSERVER.sun_rise_time(time=Time(timestamp), which='next')
-    sun_set = LOFAR_CENTER_OBSERVER.sun_set_time(time=sun_rise, which='next')
-    return sun_rise.to_datetime(), sun_set.to_datetime()
+# default angle to the horizon at which the sunset/sunrise starts and ends, as per LOFAR definition.
+SUN_SET_RISE_ANGLE_TO_HORIZON = Angle(10, unit=astropy.units.deg)
+
+def timestamps_and_stations_to_sun_rise_and_set(timestamps: [datetime], stations: [str], angle_to_horizon: Angle=SUN_SET_RISE_ANGLE_TO_HORIZON) -> dict:
+    """
+    compute sunrise, sunset, day and night of the given stations at the given timestamps
+    :param timestamps: list of datetimes, e.g. [datetime(2020, 1, 1), datetime(2020, 1, 2)]
+    :param stations: list of station names, e.g. ["CS001"]
+    :return A dict that maps station names to a nested dict that contains lists of start and end times for sunrise, sunset, etc, on each requested date.
+        E.g.
+        {"CS001":
+            {   "sunrise": [{"start": (2020, 1, 1, 6, 0, 0)), "end": (2020, 1, 1, 6, 30, 0)},
+                            {"start": (2020, 1, 2, 6, 0, 0)), "end": (2020, 1, 2, 6, 30, 0)}],
+                "sunset": [{"start": (2020, 1, 1, 18, 0, 0)), "end": (2020, 1, 1, 18, 30, 0)},
+                           {"start": (2020, 1, 2, 18, 0, 0)), "end": (2020, 1, 2, 18, 30, 0)}],
+                "day": [{"start": (2020, 1, 1, 6, 30, 0)), "end": (2020, 1, 1, 18, 00, 0)},
+                        {"start": (2020, 1, 2, 6, 30, 0)), "end": (2020, 1, 2, 18, 00, 0)}],
+                "night": [{"start": (2020, 1, 1, 18, 30, 0)), "end": (2020, 1, 2, 6, 0, 0)},
+                          {"start": (2020, 1, 2, 18,3 0, 0)), "end": (2020, 1, 3, 6, 0, 0)}],
+            }
+        }
+    """
+    return_dict = {}
+    for station in stations:
+        for timestamp in timestamps:
+            observer = create_astroplan_observer_for_station(station)
+            sunrise_start = observer.sun_rise_time(time=Time(timestamp), which='previous')
+            if sunrise_start.to_datetime().date() < timestamp.date():
+                sunrise_start = observer.sun_rise_time(time=Time(timestamp), horizon=-angle_to_horizon, which='next')
+            sunrise_end = observer.sun_rise_time(time=Time(timestamp), horizon=angle_to_horizon, which='next')
+            sunset_start = observer.sun_set_time(time=sunrise_end, horizon=angle_to_horizon, which='next')
+            sunset_end = observer.sun_set_time(time=sunrise_end, horizon=-angle_to_horizon, which='next')
+            sunrise_next_start = observer.sun_rise_time(time=sunset_end, horizon=-angle_to_horizon, which='next')
+            return_dict.setdefault(station, {}).setdefault("sunrise", []).append({"start": sunrise_start.to_datetime(), "end": sunrise_end.to_datetime()})
+            return_dict[station].setdefault("sunset", []).append({"start": sunset_start.to_datetime(), "end": sunset_end.to_datetime()})
+            return_dict[station].setdefault("day", []).append({"start": sunrise_end.to_datetime(), "end": sunset_start.to_datetime()})
+            return_dict[station].setdefault("night", []).append({"start": sunset_end.to_datetime(), "end": sunrise_next_start.to_datetime()})
+    return return_dict
 
 
 def local_sidereal_time_for_utc_and_station(timestamp: datetime = None,
diff --git a/SAS/TMSS/src/tmss/tmssapp/populate.py b/SAS/TMSS/src/tmss/tmssapp/populate.py
index d0028e00f4944da0df7d017ef81eeebcdd35ba1c..41ca47ff2037faa51c84957285efa264520921c8 100644
--- a/SAS/TMSS/src/tmss/tmssapp/populate.py
+++ b/SAS/TMSS/src/tmss/tmssapp/populate.py
@@ -82,9 +82,9 @@ def populate_test_data():
                         constraints_spec['daily']['avoid_twilight'] = unit_nr%4>1
 
                         # add the scheduling_unit_doc to a new SchedulingUnitDraft instance, and were ready to use it!
-                        scheduling_unit_draft = models.SchedulingUnitDraft.objects.create(name="UC1 %s.%s.%s.%s" % ('day' if constraints_spec['daily']['require_day'] else 'night' if constraints_spec['daily']['require_night'] else 'anytime',
-                                                                                                                    'no_tl' if constraints_spec['daily']['avoid_twilight'] else 'tl_ok',
-                                                                                                                  set_nr+1, unit_nr+1),
+                        scheduling_unit_draft = models.SchedulingUnitDraft.objects.create(name="UC1 %s.%s.%s" % ('day' if constraints_spec['daily']['require_day'] else 'night' if constraints_spec['daily']['require_night'] else 'anytime',
+                                                                                                                 'no_tl' if constraints_spec['daily']['avoid_twilight'] else 'tl_ok',
+                                                                                                                  unit_nr+1),
                                                                                           scheduling_set=scheduling_set,
                                                                                           requirements_template=strategy_template.scheduling_unit_template,
                                                                                           requirements_doc=scheduling_unit_spec,
diff --git a/SAS/TMSS/src/tmss/urls.py b/SAS/TMSS/src/tmss/urls.py
index 7e722d4cc217facbec5652268593c7946b457ebe..745ebc07401316173ceaba8a9b2fffeaf1c364d6 100644
--- a/SAS/TMSS/src/tmss/urls.py
+++ b/SAS/TMSS/src/tmss/urls.py
@@ -64,8 +64,7 @@ urlpatterns = [
     path('schemas/<str:template>/<str:name>/<str:version>/', views.get_template_json_schema, name='get_template_json_schema'),
     path('station_groups/<str:template_name>/<str:template_version>/<str:station_group>', views.get_stations_in_group, name='get_stations_in_group'), #TODO: how to make trailing slash optional?
     path('station_groups/<str:template_name>/<str:template_version>/<str:station_group>/', views.get_stations_in_group, name='get_stations_in_group'),
-    path('sun_rise_and_set/<str:timestamp>', views.get_sun_rise_and_set_at_lofar_center, name='get_sun_rise_and_set_at_lofar_center'),
-    path('todays_sun_rise_and_set', views.get_todays_sun_rise_and_set_at_lofar_center, name='get_todays_sun_rise_and_set_at_lofar_center'),
+    path('util/sun_rise_and_set', views.get_sun_rise_and_set, name='get_sun_rise_and_set'),
     path(r'util/utc', views.utc, name="system-utc"),
     path(r'util/lst', views.lst, name="conversion-lst"),
 ]
diff --git a/SAS/TMSS/test/t_conversions.py b/SAS/TMSS/test/t_conversions.py
index ccd4025f6c4c21a43d63f5ccb6a55c3b764f0963..14231c4f091c04b1f3c53b971bbf069555e6000f 100755
--- a/SAS/TMSS/test/t_conversions.py
+++ b/SAS/TMSS/test/t_conversions.py
@@ -26,6 +26,7 @@ import logging
 import requests
 import dateutil.parser
 import astropy.coordinates
+import json
 
 logger = logging.getLogger(__name__)
 logging.basicConfig(format='%(asctime)s %(levelname)s %(message)s', level=logging.INFO)
@@ -127,6 +128,43 @@ class UtilREST(unittest.TestCase):
         lon_str2 = r2.content.decode('utf8')
         self.assertNotEqual(lon_str1, lon_str2)
 
+    def test_util_sun_rise_and_set_returns_json_structure_with_defaults(self):
+        r = requests.get(BASE_URL + '/util/sun_rise_and_set', auth=AUTH)
+        self.assertEqual(r.status_code, 200)
+        r_dict = json.loads(r.content.decode('utf-8'))
+
+        # assert defaults to core and today
+        self.assertIn('CS002', r_dict.keys())
+        sunrise_start = dateutil.parser.parse(r_dict['CS002']['sunrise'][0]['start'])
+        self.assertEqual(datetime.date.today(), sunrise_start.date())
+
+    def test_util_sun_rise_and_set_considers_stations(self):
+        stations = ['CS005', 'RS305', 'DE609']
+        r = requests.get(BASE_URL + '/util/sun_rise_and_set?stations=%s' % ','.join(stations), auth=AUTH)
+        self.assertEqual(r.status_code, 200)
+        r_dict = json.loads(r.content.decode('utf-8'))
+
+        # assert station is included in response and timestamps differ
+        sunset_start_last = None
+        for station in stations:
+            self.assertIn(station, r_dict.keys())
+            sunset_start = dateutil.parser.parse(r_dict[station]['sunset'][0]['start'])
+            if sunset_start_last:
+                self.assertNotEqual(sunset_start, sunset_start_last)
+            sunset_start_last = sunset_start
+
+    def test_util_sun_rise_and_set_considers_timestamps(self):
+        timestamps = ['2020-01-01', '2020-02-22T16-00-00', '2020-3-11', '2020-01-01']
+        r = requests.get(BASE_URL + '/util/sun_rise_and_set?timestamps=%s' % ','.join(timestamps), auth=AUTH)
+        self.assertEqual(r.status_code, 200)
+        r_dict = json.loads(r.content.decode('utf-8'))
+
+        # assert all requested timestamps are included in response (sunrise on same day)
+        for i in range(len(timestamps)):
+            expected_date = dateutil.parser.parse(timestamps[i]).date()
+            response_date = dateutil.parser.parse(r_dict['CS002']['sunrise'][i]['start']).date()
+            self.assertEqual(expected_date, response_date)
+
 
 if __name__ == "__main__":
     os.environ['TZ'] = 'UTC'
diff --git a/SAS/TMSS/src/tmss/tmssapp/views.py b/SAS/TMSS/src/tmss/tmssapp/views.py
index 9d50d716fc12066c56f8b16055c20c7c761bb8fa..851a625197765c401e1cc54db50c4b33d986b2e7 100644
--- a/SAS/TMSS/src/tmss/tmssapp/views.py
+++ b/SAS/TMSS/src/tmss/tmssapp/views.py
@@ -7,13 +7,14 @@ from lofar.common.json_utils import get_default_json_object_for_schema
 from lofar.common.datetimeutils import formatDatetime
 from lofar.sas.tmss.tmss.tmssapp.adapters.parset import convert_to_parset
 from drf_yasg.utils import swagger_auto_schema
+from drf_yasg.openapi import Parameter
 from rest_framework.permissions import AllowAny
 from rest_framework.decorators import authentication_classes, permission_classes
 from django.apps import apps
 
 from datetime import datetime
 import dateutil.parser
-from lofar.sas.tmss.tmss.tmssapp.conversions import local_sidereal_time_for_utc_and_station, local_sidereal_time_for_utc_and_longitude, sun_rise_and_set_at_lofar_center
+from lofar.sas.tmss.tmss.tmssapp.conversions import local_sidereal_time_for_utc_and_station, local_sidereal_time_for_utc_and_longitude, timestamps_and_stations_to_sun_rise_and_set
 
 def subtask_template_default_specification(request, subtask_template_pk:int):
     subtask_template = get_object_or_404(models.SubtaskTemplate, pk=subtask_template_pk)
@@ -85,10 +86,24 @@ def get_stations_in_group(request, template_name:str, template_version:str, stat
                          'stations': stations})
 
 
+@permission_classes([AllowAny])
+@authentication_classes([AllowAny])
+@swagger_auto_schema(responses={200: 'An isoformat timestamp of the current UTC clock of the system'},
+                     operation_description="Get the current system time in UTC")
 def utc(request):
     return HttpResponse(datetime.utcnow().isoformat(), content_type='text/plain')
 
-
+@permission_classes([AllowAny])
+@authentication_classes([AllowAny])
+@swagger_auto_schema(responses={200: 'The LST time in hms format at the given UTC time and station or longitude'},
+                     operation_description="Get LST time for UTC time and station or longitude",
+                     manual_parameters=[Parameter(name='station', required=False, type='string', in_='query',
+                                                  description="A station names (defaults to CS002)"),
+                                        Parameter(name='timestamp', required=False, type='string', in_='query',
+                                                  description="A timestamp in isoformat (defaults to utcnow)"),
+                                        Parameter(name='longitude', required=False, type='float', in_='query',
+                                                  description="A longitude")
+                                        ])
 def lst(request):
     # Handling optional parameters via django paths in urls.py is a pain, we access them on the request directly instead.
     timestamp = request.GET.get('timestamp', None)
@@ -113,26 +128,30 @@ def lst(request):
     return HttpResponse(str(lst_lon), content_type='text/plain')
 
 
-def get_todays_sun_rise_and_set_at_lofar_center(request) -> JsonResponse:
-    '''
-    Get today's the sunrise and sunset datetime at the lofar center.
-    :returns: JSONResponse with the following contents: {"sun_rise": "2020-10-22T06:17:53Z", "sun_set": "2020-10-22T16:15:08"}
-    '''
-    return get_sun_rise_and_set_at_lofar_center(request, datetime.utcnow())
-
-
-def get_sun_rise_and_set_at_lofar_center(request, timestamp:None) -> JsonResponse:
-    '''
-    Get the sunrise and sunset datetime at the lofar center for a given day.
-    :returns: JSONResponse with the following contents: {"sun_rise": "2020-10-22T06:17:53Z", "sun_set": "2020-10-22T16:15:08"}
-    '''
-    if timestamp is None:
-        timestamp = datetime.utcnow()
-
-    if isinstance(timestamp, str):
-        timestamp = dateutil.parser.parse(timestamp)
+@permission_classes([AllowAny])
+@authentication_classes([AllowAny])
+@swagger_auto_schema(responses={200: 'A JSON object with sunrise, sunset, day and night of the given stations at the given timestamps'},
+                     operation_description="Get sunrise, sunset, day and night for stations and timestamps",
+                     manual_parameters=[Parameter(name='stations', required=False, type='string', in_='query',
+                                                  description="comma-separated list of station names"),
+                                        Parameter(name='timestamps', required=False, type='string', in_='query',
+                                                  description="comma-separated list of isoformat timestamps")])
+def get_sun_rise_and_set(request):
+    """
+    returns sunrise and sunset at the given stations and timestamps, or today at LOFAR core if none specified.
+    example request: /api/util/sun_rise_and_set?stations=CS002,CS005&timestamps=2020-05-01,2020-09-09T11-11-00
+    """
+    timestamps = request.GET.get('timestamps', None)
+    stations = request.GET.get('stations', None)
+    if timestamps is None:
+        timestamps = [datetime.utcnow()]
+    else:
+        timestamps = timestamps.split(',')
+        timestamps = [dateutil.parser.parse(timestamp) for timestamp in timestamps]  #  isot to datetime
+    if stations is None:
+        stations = ['CS002']
+    else:
+        stations = stations.split(',')
 
-    sun_rise, sun_set = sun_rise_and_set_at_lofar_center(timestamp)
-    return JsonResponse({'sun_rise': sun_rise.isoformat()+'Z',
-                         'sun_set': sun_set.isoformat()+'Z'})
+    return JsonResponse(timestamps_and_stations_to_sun_rise_and_set(timestamps, stations))
 